/*
   External Function Library for allocation with LP_SOLVE

   Copyright 2013 Tom Fiddaman, Ventana Systems, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0 

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/


//#include "c:\\vendev\\src\\vensim\\vfile.h"
//#include "c:\\vendev\\src\\vensim\\wintype.h"
#include <stdio.h>
#include <stdlib.h>

#include "lp.h"


// OK to share a single instance? I think so ...
HINSTANCE lpsolve_dll;


void lp_solve_free() // todo: where to call this on close?
{
	FreeLibrary(lpsolve_dll); 
}



HMODULE GetCurrentModule()
{ // NB: XP+ solution!
  HMODULE hModule = NULL;
  GetModuleHandleEx(
    GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS,
    (LPCTSTR)GetCurrentModule,
    &hModule);

  return hModule;
}






// load lp_solve library
// return 0 if fail to load dll
// return -1 if load succeeds, but getprocaddr calls fail
// return -2 if load succeeds, but test fails
// return 1 for success
int lp_solve_load()
{

	lprec *lp; // lp record
	HMODULE hMod ;
	int nResult;
	char strDLLFilename[MAX_PATH];
	char strDLLPath[MAX_PATH];
	
	lpsolve_dll = NULL;
	
	hMod = GetCurrentModule();
	if ( hMod != NULL )
	{
		nResult = GetModuleFileName(hMod, strDLLFilename, MAX_PATH);
		nResult = _splitpath_s(strDLLFilename,NULL, 0, strDLLPath, MAX_PATH, NULL, 0, NULL, 0);

		sprintf(strDLLFilename, "%slpsolve55.dll",strDLLPath);
		
		// first try the external function folder 
		lpsolve_dll = LoadLibrary( strDLLFilename );

	}

	//Now try the system path
	if ( lpsolve_dll == NULL )
		lpsolve_dll = LoadLibrary( "lpsolve55.dll" );

	if (!lpsolve_dll)
	{

		// try lp_solve install path instead
		lpsolve_dll = LoadLibrary( "c:\\lp_solve\\lpsolve55.dll" );

		if (!lpsolve_dll)
		{

#if defined(_DEBUG)
			int i = GetLastError();
			// todo v_error( STOP, "Unable to load lp_solve; error code is %d", i );
#endif
			return(0);
		}
	}
	
	// populate api needed
	_make_lp = (make_lp_func *) GetProcAddress(lpsolve_dll, "make_lp");
	_delete_lp = (delete_lp_func *) GetProcAddress(lpsolve_dll, "delete_lp");
	_add_constraintex = (add_constraintex_func *) GetProcAddress(lpsolve_dll, "add_constraintex");
	_write_lp = (write_lp_func *) GetProcAddress(lpsolve_dll, "write_lp");
	_set_add_rowmode = (set_add_rowmode_func *) GetProcAddress(lpsolve_dll, "set_add_rowmode");
	_set_bounds = (set_bounds_func *) GetProcAddress(lpsolve_dll, "set_bounds");
	_set_maxim = (set_maxim_func *) GetProcAddress(lpsolve_dll, "set_maxim");
	_set_obj_fnex = (set_obj_fnex_func *) GetProcAddress(lpsolve_dll, "set_obj_fnex");
	_set_verbose = (set_verbose_func *) GetProcAddress(lpsolve_dll, "set_verbose");
	_get_objective = (get_objective_func *) GetProcAddress(lpsolve_dll, "get_objective");
	_get_variables = (get_variables_func *) GetProcAddress(lpsolve_dll, "get_variables");
	_solve = (solve_func *) GetProcAddress(lpsolve_dll, "solve");
	_get_Ncolumns = (get_Ncolumns_func *) GetProcAddress(lpsolve_dll, "get_Ncolumns");
	_add_constraint = (add_constraint_func *) GetProcAddress(lpsolve_dll, "add_constraint");

	if ( ! _make_lp || ! _delete_lp || ! _add_constraintex || ! _write_lp || ! _set_add_rowmode || ! _set_bounds
		|| ! _set_maxim || ! _set_obj_fnex || ! _set_verbose || ! _get_objective || ! _get_variables || ! _solve 
		|| ! _get_Ncolumns || ! _add_constraint )
	{
		FreeLibrary(lpsolve_dll);
		return(-1);
	}

	// test
	lp = _make_lp(0,4);
	if (lp == NULL)
	{
		return(-2);
	}
	_delete_lp(lp);
	

	return(1);

}



